from pydantic_core import ValidationError


try:
    import typer
    from typing import Annotated
    from pathlib import Path
    from rich.console import Console, Group
    from rich.pretty import pprint
    from rich import print
    from rich.syntax import Syntax
    from rich.panel import Panel
    from rich.text import Text
    from pygments.styles import get_all_styles

    from .settings import UserConfig
    from .custom_typer import Typer
    from .chains import create_chain
    from .config import app as config_app

    config = UserConfig()

except ValidationError:
    print("[red]Please run `whisper init` to initialize the configuration.[/red]")

app = Typer()


@app.command(default=True)
def ask(
    prompt: Annotated[str, typer.Argument(help="The question you want to ask Whisper")],
    key: Annotated[
        str, typer.Option("-k", "--key", help="The API To use (openai, anthropic, etc)")
    ] = None,
    model: Annotated[
        str, typer.Option("-m", "--model", help="The model that Whisper should use")
    ] = None,
    theme: Annotated[
        str,
        typer.Option(
            "-t", "--theme", help="The theme that should be used for the output"
        ),
    ] = "solarized-dark",
):
    """
    The default command that takes a prompt and optional parameters to ask a question using Whisper.
    """
    console = Console(color_system="auto")

    with console.status("Thinking...", spinner="dots"):
        chain = create_chain(key, model)
        result = chain.invoke({"context": prompt})

    syntax = Syntax(
        result.snippet, lexer=result.language, line_numbers=True, padding=1, theme=theme
    )
    description_text = Text(
        result.description,
    )
    combined_content = Group(
        description_text, syntax
    )  # Group the description and syntax
    console.print(
        Panel(combined_content, title="Result", border_style="green")
    )  # Wrap combined content in a single Panel


@app.command()
def themes():
    """Prints a list of available syntax highlighting themes from Pygments."""
    pprint(list(get_all_styles()))


@app.command()
def init(
    openai_key: str = typer.Option(
        None, "--openai-key", help="The OpenAI API key to use"
    ),
    anthropic_key: str = typer.Option(
        None, "--anthropic-key", help="The Anthropic API key to use"
    ),
    show: Annotated[
        bool, typer.Option("--show/--no-show", help="Show the configuration")
    ] = True,
):
    """
    Initializes the configuration for Whisper, saving API keys to a local file.
    """
    config_path = Path.home() / ".whisper"
    config_path.mkdir(parents=True, exist_ok=True)
    config_file = config_path / "whisper.toml"

    with config_file.open("w") as f:
        toml_str = UserConfig.initialize(openai_key, anthropic_key)
        f.write(toml_str)

    if show:
        print(toml_str)

    print(f"Initialized whisper configuration in {config_file}")


app.add_typer(config_app, name="config")

if __name__ == "__main__":
    app()



import toml

from pathlib import Path
from typing import Type, Tuple

from pydantic_core import PydanticUndefinedType
from pydantic import BaseModel, Field

from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    TomlConfigSettingsSource,
)


class BaseModelConfig(BaseModel):
    api_key: str = "replace-me"
    model: str


class OpenAIConfig(BaseModelConfig):
    model: str = "gpt-3.5-turbo"


class AnthropicConfig(BaseModelConfig):
    model: str = "claude-3-5-sonnet-20240620"


class UserConfig(BaseSettings):
    model_config = SettingsConfigDict(toml_file=["~/.whisper/whisper.toml"])

    default: str = "openai"

    openai: OpenAIConfig = Field(default_factory=OpenAIConfig)
    anthropic: AnthropicConfig = Field(default_factory=AnthropicConfig)

    def get_by_dot_notation(self, key: str):
        keys = key.split(".")
        value = self
        for k in keys:
            value = getattr(value, k)
        return value

    def set_by_dot_notation(self, key: str, value: str):
        keys = key.split(".")
        v = self
        for k in keys[:-1]:
            v = getattr(v, k)
        setattr(v, keys[-1], value)

    def save(self):
        path = Path.home() / ".whisper" / "whisper.toml"
        with path.open("w") as f:
            toml.dump(self.model_dump(), f)

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        return (TomlConfigSettingsSource(settings_cls),)

    @classmethod
    def initialize(cls, openai_key: str = None, anthropic_key: str = None) -> str:
        def recurse_fields(cls_fields):
            data = {}
            for key, field in cls_fields.items():
                if issubclass(field.annotation, BaseModel):
                    data[key] = recurse_fields(field.annotation.model_fields)
                elif not isinstance(field.default, PydanticUndefinedType):
                    data[key] = field.default
            return data

        config_data = recurse_fields(cls.model_fields)

        if openai_key is not None:
            config_data["openai"]["api_key"] = openai_key
        if anthropic_key is not None:
            config_data["anthropic"]["api_key"] = anthropic_key

        return toml.dumps(config_data)



import typer
from rich.pretty import pprint
from .settings import UserConfig

app = typer.Typer()


@app.command()
def set(
    key: str = typer.Argument(
        ...,
        help="The key to set the value for.  Supports dot notation.  e.g. 'openai.api_key'",
    ),
    value: str = typer.Argument(..., help="The value to set the key to."),
):
    """
    Initialize the configuration for Whisper.
    """
    config = UserConfig()
    config.set_by_dot_notation(key, value)
    config.save()

    pprint(config)


@app.command()
def show():
    """
    Show the current configuration for Whisper.
    """
    config = UserConfig()
    pprint(config)


@app.command()
def get(
    key: str = typer.Argument(
        ...,
        help="The key to get the value for.  Supports dot notation.  e.g. 'openai.api_key'",
    ),
):
    """
    Get the current configuration for Whisper.
    """
    config = UserConfig()

    typer.echo(config.get_by_dot_notation(key))



from pydantic import BaseModel, Field
from langchain_openai import ChatOpenAI
from langchain_anthropic import ChatAnthropic
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.output_parsers import StrOutputParser

from .settings import UserConfig

config = UserConfig()


TEMPLATE = ChatPromptTemplate.from_template(
    """
You are an AI assistant that can answer questions and help with tasks.

{context}
"""
)


class CodeWhisper(BaseModel):
    language: str = Field(
        description="The programming language of the code snippet or terminal command"
    )
    description: str = Field(
        description="A description of the code snippet or terminal command"
    )
    snippet: str = Field(description="The code snippet or terminal command")


def create_chain(key: str = None, model: str = None):
    key = key or config.default
    model = model or getattr(config, key).model

    if key == "openai":
        return TEMPLATE | ChatOpenAI(
            model=model, temperature=0, openai_api_key=config.openai.api_key
        ).with_structured_output(CodeWhisper)
    elif key == "anthropic":
        return (
            TEMPLATE
            | ChatAnthropic(
                model=model, temperature=0, anthropic_api_key=config.anthropic.api_key
            )
            | StrOutputParser()
        )
    else:
        raise ValueError(f"Model {model} not found")



import typer
import click
from collections.abc import Callable, Sequence
from typing import Any
from typer.core import DEFAULT_MARKUP_MODE, MarkupMode, TyperCommand
from typer.models import CommandFunctionType, Default


class TyperDefaultCommand(typer.core.TyperCommand):
    """Type that indicates if a command is the default command."""


class TyperGroupWithDefault(typer.core.TyperGroup):
    """Use a default command if specified."""

    def __init__(
        self,
        *,
        name: str | None = None,
        commands: dict[str, click.Command] | Sequence[click.Command] | None = None,
        rich_markup_mode: MarkupMode = DEFAULT_MARKUP_MODE,
        rich_help_panel: str | None = None,
        **attrs: Any,
    ) -> None:
        super().__init__(
            name=name,
            commands=commands,
            rich_markup_mode=rich_markup_mode,
            rich_help_panel=rich_help_panel,
            **attrs,
        )
        # find the default command if any
        self.default_command = None
        if len(self.commands) > 1:
            for name, command in reversed(self.commands.items()):
                if isinstance(command, TyperDefaultCommand):
                    self.default_command = name
                    break

    def make_context(
        self,
        info_name: str | None,
        args: list[str],
        parent: click.Context | None = None,
        **extra: Any,
    ) -> click.Context:
        # if --help is specified, show the group help
        # else if default command was specified in the group and no args or no subcommand is specified, use the default command
        if (
            self.default_command
            and (not args or args[0] not in self.commands)
            and "--help" not in args
            and "--show-completion" not in args
            and "--install-completion" not in args
        ):
            args = [self.default_command] + args
        return super().make_context(info_name, args, parent, **extra)


class Typer(typer.Typer):
    """Typer with default command support."""

    def __init__(
        self,
        *,
        name: str | None = Default(None),
        invoke_without_command: bool = Default(False),
        no_args_is_help: bool = Default(False),
        subcommand_metavar: str | None = Default(None),
        chain: bool = Default(False),
        result_callback: Callable[..., Any] | None = Default(None),
        context_settings: dict[Any, Any] | None = Default(None),
        callback: Callable[..., Any] | None = Default(None),
        help: str | None = Default(None),
        epilog: str | None = Default(None),
        short_help: str | None = Default(None),
        options_metavar: str = Default("[OPTIONS]"),
        add_help_option: bool = Default(True),
        hidden: bool = Default(False),
        deprecated: bool = Default(False),
        add_completion: bool = True,
        rich_markup_mode: MarkupMode = Default(DEFAULT_MARKUP_MODE),
        rich_help_panel: str | None = Default(None),
        pretty_exceptions_enable: bool = True,
        pretty_exceptions_show_locals: bool = True,
        pretty_exceptions_short: bool = True,
    ):
        super().__init__(
            name=name,
            cls=TyperGroupWithDefault,
            invoke_without_command=invoke_without_command,
            no_args_is_help=no_args_is_help,
            subcommand_metavar=subcommand_metavar,
            chain=chain,
            result_callback=result_callback,
            context_settings=context_settings,
            callback=callback,
            help=help,
            epilog=epilog,
            short_help=short_help,
            options_metavar=options_metavar,
            add_help_option=add_help_option,
            hidden=hidden,
            deprecated=deprecated,
            add_completion=add_completion,
            rich_markup_mode=rich_markup_mode,
            rich_help_panel=rich_help_panel,
            pretty_exceptions_enable=pretty_exceptions_enable,
            pretty_exceptions_show_locals=pretty_exceptions_show_locals,
            pretty_exceptions_short=pretty_exceptions_short,
        )

    def command(
        self,
        name: str | None = None,
        *,
        cls: type[TyperCommand] | None = None,
        context_settings: dict[Any, Any] | None = None,
        help: str | None = None,
        epilog: str | None = None,
        short_help: str | None = None,
        options_metavar: str = "[OPTIONS]",
        add_help_option: bool = True,
        no_args_is_help: bool = False,
        hidden: bool = False,
        deprecated: bool = False,
        rich_help_panel: str | None = Default(None),
        default: bool = False,
    ) -> Callable[[CommandFunctionType], CommandFunctionType]:
        return super().command(
            name,
            cls=TyperDefaultCommand if default else cls,
            context_settings=context_settings,
            help=help,
            epilog=epilog,
            short_help=short_help,
            options_metavar=options_metavar,
            add_help_option=add_help_option,
            no_args_is_help=no_args_is_help,
            hidden=hidden,
            deprecated=deprecated,
            rich_help_panel=rich_help_panel,
        )
